@@ -19,15 +19,15 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from datetime import datetime, time
 import datetime as dt
 
 import urwid
 
-from ..utils import get_weekday_occurrence
-from .widgets import DateWidget, TimeWidget, NColumns, NPile, ValidatedEdit, \
-    DateConversionError, Choice, PositiveIntEdit, AlarmsEditor, NListBox, ExtendedEdit
+from ..utils import get_weekday_occurrence, get_wrapped_text
 from .calendarwidget import CalendarWidget
+from .widgets import (AlarmsEditor, Choice, DateConversionError, DateWidget,
+                      ExtendedEdit, NColumns, NListBox, NPile, PositiveIntEdit,
+                      TimeWidget, ValidatedEdit)
 
 
 class StartEnd(object):
@@ -41,9 +41,11 @@ class StartEnd(object):
 
 
 class CalendarPopUp(urwid.PopUpLauncher):
-    def __init__(self, widget, on_date_change, weeknumbers=False, firstweekday=0, keybindings=None):
+    def __init__(self, widget, on_date_change, weeknumbers=False,
+                 firstweekday=0, monthdisplay='firstday', keybindings=None):
         self._on_date_change = on_date_change
         self._weeknumbers = weeknumbers
+        self._monthdisplay = monthdisplay
         self._firstweekday = firstweekday
         self._keybindings = {} if keybindings is None else keybindings
         self.__super.__init__(widget)
@@ -70,6 +72,7 @@ class CalendarPopUp(urwid.PopUpLauncher):
                 on_change, self._keybindings, on_press,
                 firstweekday=self._firstweekday,
                 weeknumbers=self._weeknumbers,
+                monthdisplay=self._monthdisplay,
                 initial=initial_date)
             pop_up = urwid.LineBox(pop_up)
             return pop_up
@@ -89,7 +92,8 @@ class DateEdit(urwid.WidgetWrap):
     def __init__(
             self, startdt=None, dateformat='%Y-%m-%d',
             on_date_change=lambda _: None,
-            weeknumbers=False, firstweekday=0, keybindings=None,
+            weeknumbers=False, firstweekday=0, monthdisplay='firstday',
+            keybindings=None,
     ):
         datewidth = len(startdt.strftime(dateformat)) + 1
         self._dateformat = dateformat
@@ -101,13 +105,14 @@ class DateEdit(urwid.WidgetWrap):
             validate=self._validate,
             edit_text=startdt.strftime(dateformat),
             on_date_change=on_date_change)
-        wrapped = CalendarPopUp(self._edit, on_date_change, weeknumbers, firstweekday, keybindings)
+        wrapped = CalendarPopUp(self._edit, on_date_change, weeknumbers,
+                                firstweekday, monthdisplay, keybindings)
         padded = urwid.Padding(wrapped, align='left', width=datewidth, left=0, right=1)
         super().__init__(padded)
 
     def _validate(self, text):
         try:
-            _date = datetime.strptime(text, self._dateformat).date()
+            _date = dt.datetime.strptime(text, self._dateformat).date()
         except ValueError:
             return False
         else:
@@ -128,7 +133,7 @@ class DateEdit(urwid.WidgetWrap):
 
         :type date: datetime.date
         """
-        self._edit.set_edit_text(date.strftime(self._dateformat))
+        self._edit.set_edit_text(dt.date.strftime(self._dateformat))
 
 
 class StartEndEditor(urwid.WidgetWrap):
@@ -147,7 +152,7 @@ class StartEndEditor(urwid.WidgetWrap):
         :param on_end_date_change: same as for on_start_date_change, just for the
             end date
         """
-        self.allday = not isinstance(start, datetime)
+        self.allday = not isinstance(start, dt.datetime)
         self.conf = conf
         self._startdt, self._original_start = start, start
         self._enddt, self._original_end = end, end
@@ -166,7 +171,7 @@ class StartEndEditor(urwid.WidgetWrap):
 
     @property
     def startdt(self):
-        if self.allday and isinstance(self._startdt, datetime):
+        if self.allday and isinstance(self._startdt, dt.datetime):
             return self._startdt.date()
         else:
             return self._startdt
@@ -176,7 +181,7 @@ class StartEndEditor(urwid.WidgetWrap):
         try:
             return self._startdt.time()
         except AttributeError:
-            return time(0)
+            return dt.time(0)
 
     @property
     def localize_start(self):
@@ -194,7 +199,7 @@ class StartEndEditor(urwid.WidgetWrap):
 
     @property
     def enddt(self):
-        if self.allday and isinstance(self._enddt, datetime):
+        if self.allday and isinstance(self._enddt, dt.datetime):
             return self._enddt.date()
         else:
             return self._enddt
@@ -204,33 +209,33 @@ class StartEndEditor(urwid.WidgetWrap):
         try:
             return self._enddt.time()
         except AttributeError:
-            return time(0)
+            return dt.time(0)
 
     def _validate_start_time(self, text):
         try:
-            startval = datetime.strptime(text, self.conf['locale']['timeformat'])
+            startval = dt.datetime.strptime(text, self.conf['locale']['timeformat'])
             self._startdt = self.localize_start(
-                datetime.combine(self._startdt.date(), startval.time()))
+                dt.datetime.combine(self._startdt.date(), startval.time()))
         except ValueError:
             return False
         else:
             return startval
 
     def _start_date_change(self, date):
-        self._startdt = self.localize_start(datetime.combine(date, self._start_time))
+        self._startdt = self.localize_start(dt.datetime.combine(date, self._start_time))
         self.on_start_date_change(date)
 
     def _validate_end_time(self, text):
         try:
-            endval = datetime.strptime(text, self.conf['locale']['timeformat'])
-            self._enddt = self.localize_end(datetime.combine(self._enddt.date(), endval.time()))
+            endval = dt.datetime.strptime(text, self.conf['locale']['timeformat'])
+            self._enddt = self.localize_end(dt.datetime.combine(self._enddt.date(), endval.time()))
         except ValueError:
             return False
         else:
             return endval
 
     def _end_date_change(self, date):
-        self._enddt = self.localize_end(datetime.combine(date, self._end_time))
+        self._enddt = self.localize_end(dt.datetime.combine(date, self._end_time))
         self.on_end_date_change(date)
 
     def toggle(self, checkbox, state):
@@ -245,8 +250,8 @@ class StartEndEditor(urwid.WidgetWrap):
         """
 
         if self.allday is True and state is False:
-            self._startdt = datetime.combine(self._startdt, datetime.min.time())
-            self._enddt = datetime.combine(self._enddt, datetime.min.time())
+            self._startdt = dt.datetime.combine(self._startdt, dt.datetime.min.time())
+            self._enddt = dt.datetime.combine(self._enddt, dt.datetime.min.time())
         elif self.allday is False and state is True:
             self._startdt = self._startdt.date()
             self._enddt = self._enddt.date()
@@ -254,12 +259,16 @@ class StartEndEditor(urwid.WidgetWrap):
         self.widgets.startdate = DateEdit(
             self._startdt, self.conf['locale']['longdateformat'],
             self._start_date_change, self.conf['locale']['weeknumbers'],
-            self.conf['locale']['firstweekday'], self.conf['keybindings'],
+            self.conf['locale']['firstweekday'],
+            self.conf['view']['monthdisplay'],
+            self.conf['keybindings'],
         )
         self.widgets.enddate = DateEdit(
             self._enddt, self.conf['locale']['longdateformat'],
             self._end_date_change, self.conf['locale']['weeknumbers'],
-            self.conf['locale']['firstweekday'], self.conf['keybindings'],
+            self.conf['locale']['firstweekday'],
+            self.conf['view']['monthdisplay'],
+            self.conf['keybindings'],
         )
 
         if state is True:
@@ -268,25 +277,23 @@ class StartEndEditor(urwid.WidgetWrap):
             self.widgets.endtime = urwid.Text('')
         elif state is False:
             timewidth = self._timewidth + 1
-            edit = ValidatedEdit(
+            raw_start_time_widget = ValidatedEdit(
                 dateformat=self.conf['locale']['timeformat'],
                 EditWidget=TimeWidget,
                 validate=self._validate_start_time,
                 edit_text=self.startdt.strftime(self.conf['locale']['timeformat']),
             )
-            edit = urwid.Padding(
-                edit, align='left', width=self._timewidth + 1, left=1)
-            self.widgets.starttime = edit
+            self.widgets.starttime = urwid.Padding(
+                raw_start_time_widget, align='left', width=self._timewidth + 1, left=1)
 
-            edit = ValidatedEdit(
+            raw_end_time_widget = ValidatedEdit(
                 dateformat=self.conf['locale']['timeformat'],
                 EditWidget=TimeWidget,
                 validate=self._validate_end_time,
                 edit_text=self.enddt.strftime(self.conf['locale']['timeformat']),
             )
-            edit = urwid.Padding(
-                edit, align='left', width=self._timewidth + 1, left=1)
-            self.widgets.endtime = edit
+            self.widgets.endtime = urwid.Padding(
+                raw_end_time_widget, align='left', width=self._timewidth + 1, left=1)
 
         columns = NPile([
             self.checkallday,
@@ -341,7 +348,9 @@ class EventEditor(urwid.WidgetWrap):
         self.recurrenceeditor = RecurrenceEditor(
             self.event.recurobject, self._conf, event.start_local,
         )
-        self.summary = ExtendedEdit(caption='Title: ', edit_text=event.summary)
+        self.summary = urwid.AttrMap(ExtendedEdit(
+            caption=('', 'Title:       '), edit_text=event.summary), 'edit'
+        )
 
         divider = urwid.Divider(' ')
 
@@ -353,14 +362,24 @@ class EventEditor(urwid.WidgetWrap):
             self.collection._calendars[self.event.calendar],
             decorate_choice
         )
-        self.description = ExtendedEdit(
-            caption='Description: ', edit_text=self.description, multiline=True,
+        self.description = urwid.AttrMap(
+            ExtendedEdit(
+                caption=('', 'Description: '),
+                edit_text=self.description,
+                multiline=True
+            ),
+            'edit'
+        )
+        self.location = urwid.AttrMap(ExtendedEdit(
+            caption=('', 'Location:    '), edit_text=self.location), 'edit'
+        )
+        self.categories = urwid.AttrMap(ExtendedEdit(
+            caption=('', 'Categories:  '), edit_text=self.categories), 'edit'
         )
-        self.location = ExtendedEdit(caption='Location: ', edit_text=self.location)
-        self.categories = ExtendedEdit(caption='Categories: ', edit_text=self.categories)
         self.alarms = AlarmsEditor(self.event)
         self.pile = NListBox(urwid.SimpleFocusListWalker([
-            NColumns([self.summary, self.calendar_chooser], dividechars=2),
+            self.summary,
+            urwid.Columns([(12, self.calendar_chooser)]),
             divider,
             self.location,
             self.categories,
@@ -371,8 +390,8 @@ class EventEditor(urwid.WidgetWrap):
             divider,
             self.alarms,
             divider,
-            urwid.Button('Save', on_press=self.save),
-            urwid.Button('Export', on_press=self.export)
+            urwid.Columns([(12, urwid.Button('Save', on_press=self.save))]),
+            urwid.Columns([(12, urwid.Button('Export', on_press=self.export))])
         ]), outermost=True)
         self._always_save = always_save
         urwid.WidgetWrap.__init__(self, self.pile)
@@ -386,7 +405,7 @@ class EventEditor(urwid.WidgetWrap):
 
     @property
     def title(self):  # Window title
-        return 'Edit: {}'.format(self.summary.get_edit_text())
+        return 'Edit: {}'.format(get_wrapped_text(self.summary))
 
     @classmethod
     def selectable(cls):
@@ -394,13 +413,13 @@ class EventEditor(urwid.WidgetWrap):
 
     @property
     def changed(self):
-        if self.summary.get_edit_text() != self.event.summary:
+        if get_wrapped_text(self.summary) != self.event.summary:
             return True
-        if self.description.get_edit_text() != self.event.description:
+        if get_wrapped_text(self.description) != self.event.description:
             return True
-        if self.location.get_edit_text() != self.event.location:
+        if get_wrapped_text(self.location) != self.event.location:
             return True
-        if self.categories.get_edit_text() != self.event.categories:
+        if get_wrapped_text(self.categories) != self.event.categories:
             return True
         if self.startendeditor.changed or self.calendar_chooser.changed:
             return True
@@ -411,10 +430,10 @@ class EventEditor(urwid.WidgetWrap):
         return False
 
     def update_vevent(self):
-        self.event.update_summary(self.summary.get_edit_text())
-        self.event.update_description(self.description.get_edit_text())
-        self.event.update_location(self.location.get_edit_text())
-        self.event.update_categories(self.categories.get_edit_text())
+        self.event.update_summary(get_wrapped_text(self.summary))
+        self.event.update_description(get_wrapped_text(self.description))
+        self.event.update_location(get_wrapped_text(self.location))
+        self.event.update_categories(get_wrapped_text(self.categories).split(','))
 
         if self.startendeditor.changed:
             self.event.update_start_end(
@@ -560,17 +579,18 @@ class RecurrenceEditor(urwid.WidgetWrap):
         self.repetitions_edit = PositiveIntEdit(edit_text=count)
 
         until = self._rrule.get('UNTIL', [None])[0]
-        if until is None and isinstance(self._startdt, datetime):
+        if until is None and isinstance(self._startdt, dt.datetime):
             until = self._startdt.date()
         elif until is None:
             until = self._startdt
 
-        if isinstance(until, datetime):
+        if isinstance(until, dt.datetime):
             until = until.date()
         self.until_edit = DateEdit(
             until, self._conf['locale']['longdateformat'],
             lambda _: None, self._conf['locale']['weeknumbers'],
             self._conf['locale']['firstweekday'],
+            self._conf['view']['monthdisplay'],
         )
 
         self._rebuild_weekday_checks()
@@ -611,8 +631,7 @@ class RecurrenceEditor(urwid.WidgetWrap):
         keys = set(rrule.keys())
         freq = rrule.get('FREQ', [None])[0]
         unsupported_rrule_parts = {
-            'BYSECOND', 'BYMINUTE', 'BYHOUR', 'BYYEARDAY',
-            'BYWEEKNO', 'BYMONTH', 'BYSETPOS',
+            'BYSECOND', 'BYMINUTE', 'BYHOUR', 'BYYEARDAY', 'BYWEEKNO', 'BYMONTH',
         }
         if keys.intersection(unsupported_rrule_parts):
             return False
@@ -624,6 +643,8 @@ class RecurrenceEditor(urwid.WidgetWrap):
             return False
         if rrule.get('BYDAY', ['1'])[0][0] == '-':
             return False
+        if rrule.get('BYSETPOS', [1])[0] != 1:
+            return False
         if freq not in ['DAILY', 'WEEKLY', 'MONTHLY', 'YEARLY']:
             return False
         if 'BYDAY' in keys and freq == 'YEARLY':
